using System;
using System.Configuration;
using System.IO;
using Gtk;

public partial class MainWindow: Gtk.Window
{	
	private string _DeviationDrive = "";
	private string _BackupPath = "Backups";
	private string _SelectedItem = "";
	private int _SelectedModel = 0;

	private enum SelectionAction {GoUp, GoDown, DoNothing};

	// ----------------------------------------------------------------------------------------------------------------------

	public MainWindow (): base (Gtk.WindowType.Toplevel)
	{
		Build ();

		try 
		{
			// if we have a directive use it (otherwise default to "Backups")
			if (!String.IsNullOrEmpty(ConfigurationManager.AppSettings["BackupPath"]))
				_BackupPath = ConfigurationManager.AppSettings["BackupPath"];

			// create our backup store in our user space
			if (!Directory.Exists (_BackupPath))
				Directory.CreateDirectory (_BackupPath);

			// Prepare the list to display models
			Gtk.TreeViewColumn modelColumn = new Gtk.TreeViewColumn ();
			modelColumn.Title = "Model";
			Gtk.CellRendererText modelNameCell = new Gtk.CellRendererText ();
			modelColumn.PackStart (modelNameCell, true);
			treeModels.AppendColumn (modelColumn);
			modelColumn.AddAttribute (modelNameCell, "text", 0);
			treeModels.CursorChanged+= new EventHandler(treeModels_CursorChanged);
			treeModels.RowActivated += new RowActivatedHandler (treeModels_RowActivated);

			// Check for Deviation install
			CheckDrives ();
		
			// Periodically this status
			System.Timers.Timer checkTimer = new System.Timers.Timer();
			checkTimer.Elapsed += new System.Timers.ElapsedEventHandler(OnTimer);
			int checkSeconds = Convert.ToInt32(ConfigurationManager.AppSettings ["CheckDriveIntervalSeconds"]);
			if (checkSeconds > 60)
				checkSeconds = 60;
			if (checkSeconds < 5)
				checkSeconds = 5;
			checkTimer.Interval = checkSeconds * 1000;
			checkTimer.Enabled = true;
			checkTimer.AutoReset = true;

		} 
		catch (Exception ex) 
		{
			ModelFileManager.Common.WriteError("MainWindow() constructor", ex);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------

	void treeModels_RowActivated(object sender, RowActivatedArgs e)
	{
		try
		{
			string targetFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + _SelectedModel.ToString() + ".ini");

			string exeFile = ConfigurationManager.AppSettings["ModelEditor"];
			if (exeFile == "")
			{
				ModelFileManager.ViewWindow viewer = new ModelFileManager.ViewWindow(targetFile);
				viewer.Show();
			}
			else
			{
				System.Diagnostics.ProcessStartInfo startInfo = new System.Diagnostics.ProcessStartInfo();
				startInfo.FileName = exeFile;
				string argTemplate = ConfigurationManager.AppSettings["ModelEditorArgs"];
				if (argTemplate == "")
					argTemplate = "{0}";
				string args = String.Format(argTemplate, "\"" + targetFile + "\"");
				startInfo.Arguments = args;
				System.Diagnostics.Process.Start(startInfo);
			}
		}
		catch (Exception ex)
		{
			ModelFileManager.Common.MessageBoxError (this, "Operation Was NOT Successful.");			
            ModelFileManager.Common.WriteError("treeModels_RowActivated()", ex);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------

	void treeModels_CursorChanged(object sender, EventArgs e)
	{
		TreeSelection selection = (sender as TreeView).Selection;
		TreeModel model;
		TreeIter iter;

		if(selection.GetSelected(out model, out iter))
		{
			_SelectedItem = model.GetValue (iter, 0).ToString ();
			int idLength = _SelectedItem.IndexOf("-") - 1;
			string idText = _SelectedItem.Substring(0, idLength);
			_SelectedModel = Convert.ToInt32(idText);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void OnTimer(object source, System.Timers.ElapsedEventArgs e)
	{
		CheckDrives ();
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void OnDeleteEvent (object sender, DeleteEventArgs a)
	{
		Application.Quit ();
		a.RetVal = true;
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void btnBackupAll_Clicked (object sender, EventArgs e)
	{
		try
		{

			string ModelPath = System.IO.Path.Combine(_DeviationDrive, "models");
			string NewBackup = DateTime.Now.ToString("yyyyMMdd-HHmmss");
			string NewBackupPath = System.IO.Path.Combine(_BackupPath, NewBackup);
			Directory.CreateDirectory(NewBackupPath);

			// iterate through the files copying them
			string[] fileEntries = Directory.GetFiles(ModelPath);
			foreach (string fileName in fileEntries)
			{
				string onlyName = System.IO.Path.GetFileName(fileName);
				string srcfile = System.IO.Path.Combine(ModelPath, onlyName);
				string dstfile = System.IO.Path.Combine(NewBackupPath, onlyName);
				File.Copy(srcfile, dstfile, true);
			}

			ModelFileManager.Common.MessageBoxInfo(this, "Backup " + NewBackup + " was successful!");

		}
		catch (Exception ex) 
		{
			ModelFileManager.Common.MessageBoxError (this, "Backup was not successful!");
			ModelFileManager.Common.WriteError ("btnBackupAll_Clicked()", ex);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void btnExit_Click (object sender, EventArgs e)
	{
		Application.Quit ();
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void btnCopy_Clicked (object sender, EventArgs e)
	{
		try
		{
			if (_SelectedModel > 0)
			{

				// formulate source filespec
				string srcFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + _SelectedModel.ToString() + ".ini");

				// try to find an empty model slot
				string dstFile = "";
				for (int i = 1; i <= ModelFileManager.Common.MAX_MODELS; i++)
				{
					// file to test
					string TargetFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
					// retrive the model name.... if any
					string ModelName = ModelFileManager.Common.GetModelName(TargetFile);
					if (String.IsNullOrEmpty(ModelName) || ModelName == "NONE")
					{
						dstFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
						break;
					}
				}

				// if we have a destination figured out... fill it with the copy
				if (dstFile != "")
				{
					if (File.Exists(dstFile))
						File.Delete(dstFile);
					File.Copy(srcFile, dstFile);

					// refresh the list
					LoadModelNames();
				}
				else
				{
					ModelFileManager.Common.MessageBoxError(this, "Unable to find an empty model slot.");
				}
			}
			else
			{
				ModelFileManager.Common.MessageBoxError(this, "Must select a model to copy.");
			}
		}
		catch (Exception ex)
		{
			ModelFileManager.Common.MessageBoxError(this, "Operation Was NOT Successful!");
			ModelFileManager.Common.WriteError("btnCopy_Clicked()", ex);
		}
	}
	
	// ----------------------------------------------------------------------------------------------------------------------

	protected void btnImport_Click (object sender, EventArgs e)
	{
		try 
		{
			// Create and display a fileChooserDialog
			FileChooserDialog chooser = new FileChooserDialog("Please select a model file to import ...", this, FileChooserAction.Open);
			chooser.AddButton(Stock.Cancel, ResponseType.Cancel);
			chooser.AddButton(Stock.Open, ResponseType.Ok);
			chooser.Filter = new FileFilter();
			chooser.Filter.AddPattern("*.ini");

			// run the dialog
			ResponseType RetVal = (ResponseType)chooser.Run();
			if (RetVal == ResponseType.Ok)
			{
				// model file we are importing
				string srcFile = chooser.Filename;

				// try to find an empty model slot
				string dstFile = "";
				for (int i = 1; i <= ModelFileManager.Common.MAX_MODELS; i++)
				{
					// file to test
					string TargetFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
					// retrive the model name.... if any
					string ModelName = ModelFileManager.Common.GetModelName(TargetFile);
					if (String.IsNullOrEmpty(ModelName) || ModelName == "NONE")
					{
						dstFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
						break;
					}
				}

				// if we have a destination figured out... fill it with the copy
				if (dstFile != "")
				{
					// delete the destination file
					if (File.Exists(dstFile))
						File.Delete(dstFile);

					string exeFile = ConfigurationManager.AppSettings["ModelConverter"];

					// if there is no converter defined, then just copy as is
					if (String.IsNullOrEmpty(exeFile))
					{
						File.Copy(srcFile, dstFile);
					}
					else
					{
						// otherwise... confirm the conversion
						if (ModelFileManager.Common.MessageBoxYesNo(this, "Would you like to use the model converter on the import file " + srcFile + "?"))
						{
							// run the conversion and wait for it to finish
							System.Diagnostics.ProcessStartInfo startInfo = new System.Diagnostics.ProcessStartInfo();
							startInfo.FileName = exeFile;
							string argTemplate = ConfigurationManager.AppSettings["ModelConverterArgs"];
							if (argTemplate == "")
								argTemplate = "{0} {1}";
							string args = String.Format(argTemplate, "\"" + srcFile + "\"", "\"" + dstFile + "\"");
							startInfo.Arguments = args;
							using (System.Diagnostics.Process p = System.Diagnostics.Process.Start(startInfo))
							{
								p.WaitForExit();
							}
						}
						else 
						{
							File.Copy(srcFile, dstFile);
						}                                
					}
						
					LoadModelNames();
					ModelFileManager.Common.MessageBoxInfo(this, "Import Successful.");
				}
				else
				{
					ModelFileManager.Common.MessageBoxError(this, "Unable to find an empty model slot.");
				}
			} 

			chooser.Destroy();

		} 
		catch (Exception ex) 
		{
			ModelFileManager.Common.MessageBoxError(this, "Operation Was NOT Successful!");
			ModelFileManager.Common.WriteError("btnImport_Click()", ex);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void btnRestore_Clicked (object sender, EventArgs e)
	{
		string ModelPath = System.IO.Path.Combine(_DeviationDrive, "models");
		ModelFileManager.RestoreDialog rd = new ModelFileManager.RestoreDialog (ModelPath, _BackupPath);
		rd.Run ();
		if (rd.RefreshFlag)
			LoadModelNames ();
		rd.Destroy ();
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void chkShowEmpty_Toggled (object sender, EventArgs e)
	{
		LoadModelNames ();
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void btnMoveUp_Clicked (object sender, EventArgs e)
	{
		MoveModel (true);
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void btnMoveDown_Clicked (object sender, EventArgs e)
	{
		MoveModel (false);
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void btnClear_Clicked (object sender, EventArgs e)
	{
		try 
		{
			if (_SelectedModel > 0) 
			{
				if (ModelFileManager.Common.MessageBoxConfirm (this, "Are you SURE you want to clear model slot #" + _SelectedModel.ToString () + "?")) 
				{
						string srcFile = System.IO.Path.Combine (_DeviationDrive, "models", "default.ini");
						string dstFile = System.IO.Path.Combine (_DeviationDrive, "models", "model" + _SelectedModel.ToString () + ".ini");
						File.Delete (dstFile);
						File.Copy (srcFile, dstFile);
						LoadModelNames ();
				}
			} 
			else 
			{
				ModelFileManager.Common.MessageBoxError(this, "Must select a model to clear.");
			}
		} 
		catch (Exception ex) 
		{
			ModelFileManager.Common.MessageBoxError(this, "Operation Was NOT Successful!");
			ModelFileManager.Common.WriteError("btnClear_Clicked()", ex);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------

	protected void btnRename_Clicked (object sender, EventArgs e)
	{
		try 
		{

			// Make sure *something* is selected
			if (_SelectedModel > 0) 
			{

				// now get the corresponding file on the devo
				string targetFile = System.IO.Path.Combine (_DeviationDrive, "models", "model" + _SelectedModel.ToString () + ".ini");

				// Extract the file name from the selected item
				string currentName = _SelectedItem.Substring (_SelectedItem.IndexOf ("-") + 1).Trim ();

				// set up rename form and show user
				ModelFileManager.NameDialog nd = new ModelFileManager.NameDialog(currentName);
				ResponseType response = (ResponseType)nd.Run();
				if (response ==  ResponseType.Ok) 
				{

					// get the new name
					string newName = nd.CurrentName;

					// Set the model name
					ModelFileManager.Common.SetModelName (targetFile, newName);

					// Reload the list
					LoadModelNames ();
				} 
				nd.Destroy ();
			} 
			else
				ModelFileManager.Common.MessageBoxError (this, "Must Select Model First");
		} 
		catch (Exception ex) 
		{
			ModelFileManager.Common.MessageBoxError(this, "Operation Was NOT Successful!");
			ModelFileManager.Common.WriteError("btnRename_Clicked()", ex);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------
	// Routine to check all of the removeable drives for a Deviation installation.
	// ----------------------------------------------------------------------------------------------------------------------
	private void CheckDrives()
	{
		try
		{
			//System.Diagnostics.Debug.WriteLine("CheckDrives called...");

			// if we do not know of a Deviation drive...
			if (_DeviationDrive == "")
			{
				// scan each removable drive for the  ID file
				foreach (var drive in DriveInfo.GetDrives())
				{
					string FilePathTest = System.IO.Path.Combine(drive.RootDirectory.ToString(), "tx.ini");
					if (File.Exists(FilePathTest))
					{
						// Drive found!!!
						_DeviationDrive = drive.RootDirectory.ToString();
						LoadModelNames();
						break;
					}
				}
			}
			else
			{
				// if we do know of a Deviation drive, check if it is still there
				string FilePathTest = System.IO.Path.Combine(_DeviationDrive, "tx.ini");
				if (!File.Exists(FilePathTest))
					_DeviationDrive = "";
			}

			// update the GUI based on the presence of a Devo device
			EnableGUI(_DeviationDrive != "");                
		}
		catch (Exception ex)
		{
			ModelFileManager.Common.WriteError("CheckDrives()", ex);
		}
	}
	
	// ----------------------------------------------------------------------------------------------------------------------
	// Utility routine to turn the UI on and off
	// ----------------------------------------------------------------------------------------------------------------------
	private void EnableGUI(bool Enable)
	{
		Gtk.Application.Invoke (delegate {

			if (!Enable) 
			{
				Gtk.ListStore modelListStore = new Gtk.ListStore (typeof(string));
				treeModels.Model = modelListStore;
			}

			btnBackupAll.Sensitive = Enable;
			btnRestore.Sensitive = Enable;
			btnImport.Sensitive = Enable;
			btnMoveUp.Sensitive = Enable;
			btnMoveDown.Sensitive = Enable;
			btnCopy.Sensitive = Enable;
			btnClear.Sensitive = Enable;
			btnRename.Sensitive = Enable;
			chkShowEmpty.Sensitive = Enable;
		});			
	}
	
	// ----------------------------------------------------------------------------------------------------------------------
	// Refresh routine that clears and reloads the model listbox from the current Deviation install
	// ----------------------------------------------------------------------------------------------------------------------
	private void LoadModelNames(SelectionAction action = SelectionAction.DoNothing)
	{
		try 
		{

			// remember what is currently selected
			string selectedItem = "";
			if (_SelectedModel > 0) 
			{
				selectedItem = _SelectedItem;
			}

			// refill the list
			Gtk.ListStore modelListStore = new Gtk.ListStore (typeof (string));
			for (int i = 1; i < ModelFileManager.Common.MAX_MODELS; i++)
			{
				string TargetFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
				string ModelName = ModelFileManager.Common.GetModelName(TargetFile);
				if (!String.IsNullOrEmpty(ModelName))
				{
					if (ModelName == "NONE")
					{
						if (chkShowEmpty.Active)
							modelListStore.AppendValues (i.ToString() + " -");
					}
					else
					{
						modelListStore.AppendValues (i.ToString() + " - " + ModelName);
					}
				}
			}
			treeModels.Model = modelListStore;

			// re-select the last selected model
			if (action != SelectionAction.DoNothing) 
			{
				if (selectedItem != "") 
				{
					
					int modelNo = Convert.ToInt32(selectedItem.Substring(0, selectedItem.IndexOf("-")));

//					string modelName = selectedItem.Substring(selectedItem.IndexOf("-") + 1).Trim();
//					string newItem = "";
//					if (action == SelectionAction.GoDown)
//					{
//						newItem = (modelNo + 1).ToString() + " - " + modelName;
//					} 
//					else 
//					{
//						newItem = (modelNo - 1).ToString() + " - " + modelName;
//					}

					TreeIter iter;
					if (action == SelectionAction.GoDown)
					{
						treeModels.Model.GetIterFromString (out iter, modelNo.ToString());
					} 
					else 
					{
						treeModels.Model.GetIterFromString (out iter, (modelNo - 2).ToString());
					}
					treeModels.Selection.SelectIter (iter);

					treeModels.GrabFocus();
				}
			} 
			else 
				ClearSelection();
		} 
		catch (Exception ex) 
		{
			ModelFileManager.Common.WriteError ("LoadModelNames()", ex);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------
	// Routine that renames the model files in such a way, that the specified model is moved up or down
	// ----------------------------------------------------------------------------------------------------------------------
	private void MoveModel(bool GoingUp)
	{
		try
		{
			if (_SelectedModel > 0) 
			{

				if ((GoingUp && (_SelectedModel > 1)) || (!GoingUp && (_SelectedModel < ModelFileManager.Common.MAX_MODELS)))
				{
					// copy the starting file to a temp name
					string srcFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + _SelectedModel.ToString() + ".ini");
					string dstFile = System.IO.Path.Combine(_DeviationDrive, "models", "temp.ini");
					File.Move(srcFile, dstFile);

					// copy the replaced file to the src file location
					dstFile = srcFile;
					if (GoingUp)
						srcFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + (_SelectedModel - 1).ToString() + ".ini");
					else
						srcFile = System.IO.Path.Combine(_DeviationDrive, "models", "model" + (_SelectedModel + 1).ToString() + ".ini");
					if (!File.Exists(srcFile))
					{
						string templateFile = System.IO.Path.Combine(_DeviationDrive, "models", "default.ini");
						File.Copy(templateFile, srcFile);
					}
					File.Move(srcFile, dstFile);

					// finally name the temp name to the replaced file
					dstFile = srcFile;
					srcFile = System.IO.Path.Combine(_DeviationDrive, "models", "temp.ini");
					File.Move(srcFile, dstFile);

					// refresh the list
					if (GoingUp)
						LoadModelNames(SelectionAction.GoUp);
					else
						LoadModelNames(SelectionAction.GoDown);
				}

			}
			else 
			{
				ModelFileManager.Common.MessageBoxError(this, "Must select a model to move.");
			}

		}
		catch (Exception ex)
		{
			ModelFileManager.Common.MessageBoxError(this, "Operation Was NOT Successful!");
			ModelFileManager.Common.WriteError ("MoveModel()", ex);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------
	// Clears the selection
	// ----------------------------------------------------------------------------------------------------------------------
	private void ClearSelection()
	{
		_SelectedItem = "";
		_SelectedModel = 0;
	}

}
